--[[
	Blessed
	by Ailae of Emeriss-EU
	
	Credits to zork (of Roth UI) for texture.
]]

local addon, Blessed = ...
local pguid

-- defaults
local defaults = {
	framePoint = "CENTER",
	xOfs = 0,
	yOfs = -150,
	locked = true,
	spells = {
		[1044] = { -- Hand of Freedom
			enabled = true,
			duration = 6, 
		},
		[1022] = { -- Hand of Protection
			enabled = true,
			duration = 10,
		},
		[6940] = { -- Hand of Sacrifice
			enabled = true,
			duration = 12,
		},
		[1038] = { -- Hand of Salvation
			enabled = true,
			duration = 10,
		},
		[53563] = {  -- Light's Beacon
			enabled = true,
			duration = 300,
		},
		[31842] = { -- Divine Favor
			enabled = false,
			duration = 20,
		},
		[54428] = { -- Divine Plea
			enabled = true,
			duration = 9,
		},
		[53657] = { -- Judgement of the Pure (3/3)
			enabled = true,
			duration = 60,
		},
		[20925] = { -- Holy Shield
			enabled = true,
			duration = 10, 
		},
		[31821] = { -- Aura Mastery 
			enabled = false,
			duration = 6,
		},
		[84963] = { -- Inquisition 
			enabled = true,
			duration = 12,
		},
		[88819] = { -- Daybreak
			enabled = false,
			duration = 12,
		},
		[85433] = { -- Sacred Duty
			enabled = true,
			duration = 10, 
		},
		[94686] = { -- Crusader
			enabled = true,
			duration = 15, 
		},
		[54149] = { -- Infusion of Light	
			enabled = true,
			duration = 15,
		},
		[105742] = { -- Saint's Vigor
			enabled = false,
			duration = 15,
		},
		[85696] = { -- Zealotry
			enabled = false,
			duration = 20,
		},
		[31884] = { -- Avenging Wrath
			enabled = false,
			duration = 20,
		},
		[86698] = { -- Guardian of Ancient Kings (Retribution)
			enabled = false,
			duration = 30,
		},
		[86659] = { -- Guardian of Ancient Kings (Protection)
			enabled = false,
			duration = 30,
		},		
		[86669] = { -- Guardian of Ancient Kings (Holy)
			enabled = false,
			duration = 30,
		},		
	}
}
local db

-- cosmetics
local iconSize = 36

-- Add a prefix
local function Print(...)
	print("|cff00aaffBlessed:|r", ...)
end

local INQUISITION = GetSpellInfo(84963)
local colors = setmetatable({
	DEATHKNIGHT = "c41e3a",
	DRUID = "ff7c0a",
	HUNTER = "aad372",	
	MAGE = "68ccef",
	PALADIN = "f48cba",
	PRIEST = "ffffff",
	ROGUE = "fff468",
	SHAMAN = "0070dd",
	WARLOCK = "9382c9",
	WARRIOR = "c69b6d",
	}, { __index = function()
		return "ffffff"
	end
})

-- Empty tables for our frames
local frames = {}
local framePool = {}

function Blessed:Init()
	local frame = self.frame
	frame:SetBackdrop({
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		insets = {top = 0, left = 0, bottom = 0, right = 0},
	})
	frame:SetBackdropColor(1, 1, 1, 0)

	frame:SetWidth(iconSize)
	frame:SetHeight(iconSize)
	frame:SetPoint(db.framePoint, db.xOfs, db.yOfs)

	frame:SetMovable(true)

	frame:SetScript("OnMouseDown", function()
		if(IsShiftKeyDown() and not db.locked) then
			frame:ClearAllPoints()
			frame:StartMoving()
		end
	end)

	frame:SetScript("OnMouseUp", function()
		frame:StopMovingOrSizing()
		db.framePoint, _, _, db.xOfs, db.yOfs = frame:GetPoint()
	end)

	local text = frame:CreateFontString(nil, "OVERLAY")
	text:SetFontObject(SystemFont_Shadow_Med1)
	text:SetPoint("TOP", frame, "TOP", 0, 16)
	text:SetTextColor(1, 0.82, 0)
	frame.header = text
end

function Blessed:ToggleLock()
	if db.locked then
		self.frame.header:SetText(addon)
		self.frame:SetBackdropColor(1, 1, 1, 0.25)
	else 
		self.frame.header:SetText("")
		self.frame:SetBackdropColor(1, 1, 1, 0)
	end
	self.frame:EnableMouse(db.locked)
	db.locked = not db.locked
end

function Blessed:RestoreDefaultSpells()
	db.spells = {}
	
	for k,v in pairs(defaults.spells) do
		db.spells[k] = v
	end
	
	Print("Spells have been reset to default.")
end

function Blessed:PLAYER_LOGIN()
	BlessedDB = BlessedDB or defaults
	db = BlessedDB
	if not db.spells then
		self:RestoreDefaultSpells()
	end
	
	-- If new spells have been added, add them to our savedvars
	for k,v in pairs(defaults.spells) do
		if not db.spells[k] then
			db.spells[k] = v
		end
	end
		
	-- Force lock on login
	if not db.locked then db.locked = true end
	
	local class = select(2, UnitClass("player"))
	if class == "PALADIN" then -- only do stuff if player is a paladin
		self:RegisterEvent("PLAYER_ENTERING_WORLD")
		self:Init()
		
		SLASH_Blessed1 = "/blessed"
		SlashCmdList.Blessed = function(input)
			local input = string.lower(input)
			if (input == "lock") then
				self:ToggleLock()
			else
				InterfaceOptionsFrame_OpenToCategory(self.panel)
			end
		end
	else
		Print("This addon has no use for any other class than a Paladin, you should consider disabling it for this character.")
	end	
end

-- Only activate buff-tracking if the player is a paladin
function Blessed:PLAYER_ENTERING_WORLD()
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	
	pguid = UnitGUID("player")
	
	-- this is to make sure talents are loaded or if we need
	-- to delay the check
	if (GetNumTalentTabs() == 0) then
		self:RegisterEvent("PLAYER_ALIVE")
	else
		self:UpdateDurations()
	end	
	
	self.PLAYER_ALIVE = self.UpdateDurations
	
	self:RegisterEvent("CHARACTER_POINTS_CHANGED")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	self:RegisterEvent("GLYPH_ADDED")
end

-- Check talents for updates (Hand of Freedom)
function Blessed:UpdateDurations()
	if not db.spells[1044] or not db.spells[1044].enabled then return end
	
	-- Hand of Freedom
	local rank, maxRank = select(5, GetTalentInfo(3, 4))
	if rank == maxRank then
		db.spells[1044].duration = 10
	else
		db.spells[1044].duration = 6
	end
	
	-- Divine Favor
	local timer = 20
	for i=1, NUM_GLYPH_SLOTS do
		local glyph = select(4, GetGlyphSocketInfo(i, nil))
		if glyph == 54937 then timer = 30 end
	end
	db.spells[31842].duration = timer

	self:UnregisterEvent("PLAYER_ALIVE")
end

-- Perform checks for all these events
Blessed.CHARACTER_POINTS_CHANGED = Blessed.UpdateDurations
Blessed.ACTIVE_TALENT_GROUP_CHANGED = Blessed.UpdateDurations
Blessed.GLYPH_ADDED = Blessed.UpdateDurations

-- Handle stuff, plx
function Blessed:COMBAT_LOG_EVENT_UNFILTERED(timestamp, combatEvent, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlagsTwo, destGUID, destName, destFlags, destFlagsTwo, spellID, ...)
	if sourceGUID ~= pguid or not db.spells[spellID] or not db.spells[spellID].enabled then return end
	
	if (combatEvent == "SPELL_AURA_REMOVED") then
		self:RemoveFrame(spellID, false)
	elseif (combatEvent == "SPELL_AURA_APPLIED") then
		-- Get rid of Beacon, to avoid duplicates due to being in different realms
		if spellID == 53563 then
			self:RemoveFrame(spellID)
		end
		self:SetupIcon(spellID, destName, destGUID)
	elseif (combatEvent == "SPELL_AURA_REFRESH") then
		self:RemoveFrame(spellID, true)
		self:SetupIcon(spellID, destName, destGUID)
	end
end

-- Reanchor frames
function Blessed:UpdateFrames()
	if #frames == 0 then return end
	
	for i, frame in pairs(frames) do
		frame:ClearAllPoints()
		if i > 1 then
			frame:SetPoint("LEFT", frames[i - 1], "RIGHT", 15, 0)
		else
			frame:SetPoint("CENTER", self.frame, "CENTER")
		end
	end
end

-- Remove bar
function Blessed:RemoveFrame(spellID)
	if #frames == 0 then return end
	
	for i, frame in pairs(frames) do
		if frame.id == spellID then
			frame:Hide()
			table.remove(frames, i)
			table.insert(framePool, frame)
			self:UpdateFrames()
		end
	end
end

-- Update barinfo
local function update(self, elapsed)
	self.duration = self.duration - elapsed
	self.update = self.update - elapsed
	if self.update > 0 then return end
	
	self.update = self.update + 1
		
	if self.duration > 60 then
		self.timer:SetFormattedText("%.0fm", ceil(self.duration/60))
	else
		self.timer:SetFormattedText("%.0f", floor(self.duration))
	end
	
	if self.duration <= 0 then		
		Blessed:RemoveFrame(self.id)
	elseif self.duration <= 5 then 
		self.timer:SetTextColor(1, 0, 0)
	end
end

-- Framemananger, return a new frame if there's no old frame available 
function Blessed:GetFrame()
	if (#framePool > 0) then
		return table.remove(framePool, 1)
	end
	
	local frame = CreateFrame("Frame", nil, UIParent)
	frame:SetSize(iconSize, iconSize)
	frame:SetPoint("CENTER", self.frame)
	
	-- Icon
	frame.icon = frame:CreateTexture(nil, "ARTWORK")
	frame.icon:SetSize(iconSize, iconSize)
	frame.icon:SetTexCoord(0.03, 0.97, 0.02, 0.97)
	frame.icon:SetAllPoints(frame)
	
	-- Targettext
	frame.target = frame:CreateFontString(nil, "OVERLAY")
	frame.target:SetFontObject(SystemFont_Shadow_Med1)
	frame.target:SetPoint("BOTTOM", frame, "TOP", 0, 5)
	frame.target:SetTextColor(1, 1, 1) 
	frame.target:SetShadowOffset(1, -1)
	
	-- Timertext
	frame.timer = frame:CreateFontString(nil, "OVERLAY")
	frame.timer:SetFontObject(SystemFont_Shadow_Large)
	frame.timer:SetPoint("TOP", frame, "BOTTOM", 0, -5)
	frame.timer:SetTextColor(1, 1, 1)
	frame.timer:SetShadowOffset(1, -1)	
	
	-- Cooldown-model (beta)
	frame.cd = CreateFrame("Cooldown", nil, frame, "CooldownFrameTemplate")
	-- frame.cd:SetSize(iconSize, iconSize)
	frame.cd:SetPoint("TOPLEFT", frame, "TOPLEFT", 2, -2)
	frame.cd:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -2, 2)
	frame.cd.noCooldownCount = true -- prevent OmniCC if present
	
	-- Border
	frame.border = frame:CreateTexture(nil, "OVERLAY")
	frame.border:SetParent(frame)
	frame.border:SetTexture('Interface\\AddOns\\Blessed\\simplesquare_roth')
	frame.border:SetPoint('TOPRIGHT', frame, 2, 2)
	frame.border:SetPoint('BOTTOMLEFT', frame, -2, -2)
	frame.border:SetVertexColor(0.3, 0.3, 0.3)
	
	return frame
end

-- Setup a timer
function Blessed:SetupIcon(spellID, targetName, targetGUID)
	local frame = self:GetFrame()
	local duration = db.spells[spellID].duration
	
	if spellID == 84963 then
		duration = select(6, UnitAura("player", INQUISITION))
	end
	
	local icon = select(3, GetSpellInfo(spellID))
	local class = select(2, GetPlayerInfoByGUID(targetGUID))
		
	frame.icon:SetTexture(icon)
	frame.target:SetFormattedText("|cff%s%s|r", colors[class], targetGUID ~= pguid and string.sub(targetName, 0, 3) or "You!")

	if duration > 60 then
		frame.timer:SetFormattedText("%.0fm", ceil(duration/60))
	else
		frame.timer:SetFormattedText("%.0f", floor(duration))
	end	
	frame.timer:SetTextColor(1, 1, 1)
	
	frame.duration = duration
	frame.id = spellID
	frame.update = 1
	frame.cd:SetCooldown(GetTime(), duration)
	table.insert(frames, frame)

	frame:SetScript("OnUpdate", update)	
	
	self:UpdateFrames()
	frame:Show()
end

Blessed.frame = CreateFrame("Frame", nil, UIParent)
Blessed.frame:SetScript("OnEvent", function(self, event, ...) return Blessed[event] and Blessed[event](Blessed, ...) end)

function Blessed:RegisterEvent(event) return self.frame:RegisterEvent(event) end
function Blessed:UnregisterEvent(event) return self.frame:UnregisterEvent(event) end

Blessed:RegisterEvent("PLAYER_LOGIN")