--[[
	Blessed Options
	by Ailae of Emeriss-EU
]]

local addon, Blessed = ...
local db 

local title, desc = select(2, GetAddOnInfo(addon))
local f = CreateFrame("Frame", nil, InterfaceOptionsFramePanelContainer)
Blessed.panel = f
f.name = title
f:Hide()

f:SetScript("OnShow", function(self)
	db = BlessedDB
	local items = {}
	
	-- Header
	local header = self:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	header:SetPoint("TOPLEFT", 16, -16)
	header:SetText(title)
	
	-- Tagline
	local tag = self:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	tag:SetPoint("TOPLEFT", header, "BOTTOMLEFT", 0, -5)
	tag:SetText(desc)
	
	local function Checkers(name, text, tooltip, tooltipText)
		local check = CreateFrame("CheckButton", string.format("%s%sCheck", addon, name), f, "InterfaceOptionsCheckButtonTemplate")
		check.Text:SetText(text)
		check.tooltipText = tooltip
		check.tooltipRequirement = tooltipText
		return check
	end
	
	local lock = Checkers("Lock", "Lock", "Toggle lock", "Click to toggle lock. When unlocked, press down SHIFT and drag the square to position the frame. This setting will reset to toggled on login.")
	lock:SetPoint("TOPLEFT", tag, "BOTTOMLEFT", 0, -20)
	lock:SetChecked(db.locked)
	lock:SetScript("OnClick", function()
		Blessed:ToggleLock()
	end)
	
	local lockTip = lock:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	lockTip:SetPoint("TOPLEFT", lock, "BOTTOMLEFT", 0, -5)
	lockTip:SetText("Tip: It's also possible to toggle the lock via the |cffffffff/blessed lock|r slash-command.")
	
	local reset = CreateFrame("Button", string.format("%sReset", addon), f, "OptionsButtonTemplate")
	reset:SetText("Reset spells")
	reset:SetWidth(reset:GetTextWidth() * 2)
	reset:SetPoint("TOPLEFT", lockTip, "BOTTOMLEFT", 0, -5)
	reset:SetScript("OnClick", function() 
		Blessed:RestoreDefaultSpells()
		for i=1,#items do
			local item = items[i]
			item:SetChecked(db.spells[item.spellId].enabled)
		end
	end)
	
	for k,v in pairs(db.spells) do
		local n = GetSpellInfo(k)
		local spell = Checkers(k, n, n, "Current duration is |cff00ff00" .. v.duration .. "|r seconds.")
		if (#items < 10) then
			spell:SetPoint("TOPLEFT", reset, "BOTTOMLEFT", 0, -20 * (#items+1))
		else 
			spell:SetPoint("TOPLEFT", reset, "BOTTOMLEFT", 225, -20 * (#items-9))
		end
		
		spell:SetChecked(v.enabled)
		spell:SetScript("OnClick", function()
			db.spells[k].enabled = not db.spells[k].enabled
		end)
		spell.spellId = k
		table.insert(items, spell)
	end
	
	self:SetScript("OnShow", nil)
end)

InterfaceOptions_AddCategory(f)